@@ -1,5 +1,3 @@
-from __future__ import print_function
-
 import argparse
 import copy
 from distutils import ccompiler
@@ -7,7 +5,6 @@ from distutils import errors
 from distutils import msvccompiler
 from distutils import sysconfig
 from distutils import unixccompiler
-import glob
 import os
 from os import path
 import shutil
@@ -16,56 +13,104 @@ import sys
 import pkg_resources
 import setuptools
 from setuptools.command import build_ext
-from setuptools.command import sdist
 
 from install import build
-from install.build import PLATFORM_DARWIN
 from install.build import PLATFORM_LINUX
 from install.build import PLATFORM_WIN32
 
 
-required_cython_version = pkg_resources.parse_version('0.28.0')
+# Cython requirements (minimum version and versions known to be broken).
+# Note: this must be in sync with setup_requires defined in setup.py.
+required_cython_version = pkg_resources.parse_version('0.29.22')
 ignore_cython_versions = [
 ]
 use_hip = bool(int(os.environ.get('CUPY_INSTALL_USE_HIP', '0')))
 
-MODULES = [
-    {
+
+# The value of the key 'file' is a list that contains extension names
+# or tuples of an extension name and a list of other souces files
+# required to build the extension such as .cpp files and .cu files.
+#
+#   <extension name> | (<extension name>, a list of <other source>)
+#
+# The extension name is also interpreted as the name of the Cython
+# source file required to build the extension with appending '.pyx'
+# file extension.
+MODULES = []
+
+cuda_files = [
+    'cupy_backends.cuda.api.driver',
+    'cupy_backends.cuda.api.runtime',
+    'cupy_backends.cuda.libs.cublas',
+    'cupy_backends.cuda.libs.curand',
+    'cupy_backends.cuda.libs.cusparse',
+    'cupy_backends.cuda.libs.nvrtc',
+    'cupy_backends.cuda.libs.profiler',
+    'cupy_backends.cuda.stream',
+    'cupy.core._accelerator',
+    'cupy.core._carray',
+    'cupy.core._cub_reduction',
+    'cupy.core._dtype',
+    'cupy.core._fusion_kernel',
+    'cupy.core._fusion_thread_local',
+    'cupy.core._fusion_trace',
+    'cupy.core._fusion_variable',
+    'cupy.core._kernel',
+    'cupy.core._memory_range',
+    'cupy.core._optimize_config',
+    'cupy.core._reduction',
+    'cupy.core._routines_binary',
+    'cupy.core._routines_indexing',
+    'cupy.core._routines_linalg',
+    'cupy.core._routines_logic',
+    'cupy.core._routines_manipulation',
+    'cupy.core._routines_math',
+    'cupy.core._routines_sorting',
+    'cupy.core._routines_statistics',
+    'cupy.core._scalar',
+    'cupy.core.core',
+    'cupy.core.flags',
+    'cupy.core.internal',
+    'cupy.core.fusion',
+    'cupy.core.new_fusion',
+    'cupy.core.raw',
+    'cupy.cuda.common',
+    'cupy.cuda.cufft',
+    'cupy.cuda.device',
+    'cupy.cuda.memory',
+    'cupy.cuda.memory_hook',
+    'cupy.cuda.pinned_memory',
+    'cupy.cuda.function',
+    'cupy.cuda.stream',
+    'cupy.cuda.texture',
+    'cupy.fft._cache',
+    'cupy.lib._polynomial',
+    'cupy._util'
+]
+
+if use_hip:
+    MODULES.append({
+        # TODO(leofang): call this "rocm" or "hip" to avoid confusion?
         'name': 'cuda',
-        'file': [
-            'cupy.core._dtype',
-            'cupy.core._kernel',
-            'cupy.core._memory_range',
-            'cupy.core._routines_indexing',
-            'cupy.core._routines_logic',
-            'cupy.core._routines_manipulation',
-            'cupy.core._routines_math',
-            'cupy.core._routines_sorting',
-            'cupy.core._routines_statistics',
-            'cupy.core._scalar',
-            'cupy.core.core',
-            'cupy.core.dlpack',
-            'cupy.core.flags',
-            'cupy.core.internal',
-            'cupy.core.fusion',
-            'cupy.core.raw',
-            'cupy.cuda.cublas',
-            'cupy.cuda.cufft',
-            'cupy.cuda.curand',
-            'cupy.cuda.cusparse',
-            'cupy.cuda.device',
-            'cupy.cuda.driver',
-            'cupy.cuda.memory',
-            'cupy.cuda.memory_hook',
-            'cupy.cuda.nvrtc',
-            'cupy.cuda.pinned_memory',
-            'cupy.cuda.profiler',
-            'cupy.cuda.function',
-            'cupy.cuda.stream',
-            'cupy.cuda.runtime',
-            'cupy.cuda.texture',
-            'cupy.util',
+        'file': cuda_files,
+        'include': [
+            'hip/hip_runtime_api.h',
+            'hip/hiprtc.h',
+            'hipblas.h',
+            'hiprand/hiprand.h',
         ],
+        'libraries': [
+            'amdhip64',  # was hiprtc and hip_hcc before ROCm 3.8.0
+            'hipblas',
+            'hiprand',
+        ],
+        'check_method': build.check_hip_version,
+        'version_method': build.get_hip_version,
+    })
+else:
+    MODULES.append({
+        'name': 'cuda',
+        'file': cuda_files,
         'include': [
             'cublas_v2.h',
             'cuda.h',
@@ -87,11 +132,22 @@ MODULES = [
         ],
         'check_method': build.check_cuda_version,
         'version_method': build.get_cuda_version,
-    },
-    {
+    })
+
+if use_hip:
+    MODULES.append({
         'name': 'cusolver',
         'file': [
-            'cupy.cuda.cusolver',
+            'cupy_backends.cuda.libs.cusolver',
+        ],
+        'include': [],
+        'libraries': [],
+    })
+else:
+    MODULES.append({
+        'name': 'cusolver',
+        'file': [
+            'cupy_backends.cuda.libs.cusolver',
         ],
         'include': [
             'cusolverDn.h',
@@ -100,11 +156,13 @@ MODULES = [
             'cusolver',
         ],
         'check_method': build.check_cuda_version,
-    },
-    {
+    })
+
+if not use_hip:
+    MODULES.append({
         'name': 'cudnn',
         'file': [
-            'cupy.cuda.cudnn',
+            'cupy_backends.cuda.libs.cudnn',
             'cupy.cudnn',
         ],
         'include': [
@@ -115,8 +173,9 @@ MODULES = [
         ],
         'check_method': build.check_cudnn_version,
         'version_method': build.get_cudnn_version,
-    },
-    {
+    })
+
+    MODULES.append({
         'name': 'nccl',
         'file': [
             'cupy.cuda.nccl',
@@ -129,8 +188,9 @@ MODULES = [
         ],
         'check_method': build.check_nccl_version,
         'version_method': build.get_nccl_version,
-    },
-    {
+    })
+
+    MODULES.append({
         'name': 'nvtx',
         'file': [
             'cupy.cuda.nvtx',
@@ -142,35 +202,13 @@ MODULES = [
             'nvToolsExt' if not PLATFORM_WIN32 else 'nvToolsExt64_1',
         ],
         'check_method': build.check_nvtx,
-    },
-    {
-        # The value of the key 'file' is a list that contains extension names
-        # or tuples of an extension name and a list of other souces files
-        # required to build the extension such as .cpp files and .cu files.
-        #
-        #   <extension name> | (<extension name>, a list of <other source>)
-        #
-        # The extension name is also interpreted as the name of the Cython
-        # source file required to build the extension with appending '.pyx'
-        # file extension.
-        'name': 'thrust',
-        'file': [
-            ('cupy.cuda.thrust', ['cupy/cuda/cupy_thrust.cu']),
-        ],
-        'include': [
-            'thrust/device_ptr.h',
-            'thrust/sequence.h',
-            'thrust/sort.h',
-        ],
-        'libraries': [
-            'cudart',
-        ],
-        'check_method': build.check_cuda_version,
-    },
-    {
+    })
+
+    MODULES.append({
         'name': 'cutensor',
         'file': [
-            'cupy.cuda.cutensor',
+            'cupy_backends.cuda.libs.cutensor',
+            'cupy.cutensor',
         ],
         'include': [
             'cutensor.h',
@@ -178,12 +216,12 @@ MODULES = [
         'libraries': [
             'cutensor',
             'cublas',
-            'cudart',
         ],
         'check_method': build.check_cutensor_version,
         'version_method': build.get_cutensor_version,
-    },
-    {
+    })
+
+    MODULES.append({
         'name': 'cub',
         'file': [
             ('cupy.cuda.cub', ['cupy/cuda/cupy_cub.cu']),
@@ -194,52 +232,67 @@ MODULES = [
         'libraries': [
             'cudart',
         ],
-        'check_method': build.check_cuda_version,
-    },
-]
+        'check_method': build.check_cub_version,
+        'version_method': build.get_cub_version,
+    })
+else:
+    MODULES.append({
+        'name': 'cub',
+        'file': [
+            ('cupy.cuda.cub', ['cupy/cuda/cupy_cub.cu']),
+        ],
+        'include': [
+            'hipcub/hipcub_version.hpp',  # dummy
+        ],
+        'libraries': [
+            'amdhip64',  # was hiprtc and hip_hcc before ROCm 3.8.0
+        ],
+        'check_method': build.check_cub_version,
+        'version_method': build.get_cub_version,
+    })
 
+if bool(int(os.environ.get('CUPY_SETUP_ENABLE_THRUST', 1))):
+    if use_hip:
+        MODULES.append({
+            'name': 'thrust',
+            'file': [
+                ('cupy.cuda.thrust', ['cupy/cuda/cupy_thrust.cu']),
+            ],
+            'include': [
+                'thrust/version.h',
+            ],
+            'libraries': [
+                'amdhip64',  # was hiprtc and hip_hcc before ROCm 3.8.0
+            ],
+        })
+    else:
+        MODULES.append({
+            'name': 'thrust',
+            'file': [
+                ('cupy.cuda.thrust', ['cupy/cuda/cupy_thrust.cu']),
+            ],
+            'include': [
+                'thrust/device_ptr.h',
+                'thrust/sequence.h',
+                'thrust/sort.h',
+            ],
+            'libraries': [
+                'cudart',
+            ],
+            'check_method': build.check_thrust_version,
+            'version_method': build.get_thrust_version,
+        })
 
-def convert_modules_for_hip():
-    global MODULES
-    if len(MODULES) == 3:
-        return
-    for m in MODULES:
-        if m['name'] == 'thrust':
-            mod_thrust = m
-    MODULES = MODULES[:2]
-    MODULES.append(mod_thrust)
-    mod_cuda = MODULES[0]
-    mod_cuda['include'] = [
-        'hip/hip_runtime_api.h',
-        'hip/hiprtc.h',
-        'hipblas.h',
-        'hiprand/hiprand.h',
-        #        'hipsparse.h',
-        #        'cuda_profiler_api.h',
-        #        'cufft.h',
-    ]
-    mod_cuda['libraries'] = [
-        'hiprtc',
-        'hip_hcc',
-        'hipblas',
-        'hiprand',
-        #        'hipsparse',
-        #        'cufft',
-    ]
-    del mod_cuda['version_method']
-    del mod_cuda['check_method']
-    mod_cusolver = MODULES[1]
-    mod_cusolver['include'] = []
-    mod_cusolver['libraries'] = []
-    del mod_cusolver['check_method']
-    mod_thrust['include'] = [
-        'thrust/version.h',
-    ]
-    mod_thrust['libraries'] = [
-        'hiprtc',
-        'hip_hcc',
-    ]
-    del mod_thrust['check_method']
+MODULES.append({
+    'name': 'dlpack',
+    'file': [
+        'cupy.core.dlpack',
+    ],
+    'include': [
+        'cupy/dlpack/dlpack.h',
+    ],
+    'libraries': [],
+})
 
 
 def ensure_module_file(file):
@@ -256,14 +309,7 @@ def module_extension_name(file):
 def module_extension_sources(file, use_cython, no_cuda):
     pyx, others = ensure_module_file(file)
     base = path.join(*pyx.split('.'))
-    if use_cython:
-        pyx = base + '.pyx'
-        if not os.path.exists(pyx):
-            use_cython = False
-            print(
-                'NOTICE: Skipping cythonize as {} does not exist.'.format(pyx))
-    if not use_cython:
-        pyx = base + '.cpp'
+    pyx = base + ('.pyx' if use_cython else '.cpp')
 
     # If CUDA SDK is not available, remove CUDA C files from extension sources
     # and use stubs defined in header files.
@@ -313,6 +359,7 @@ def preconfigure_modules(compiler, settings):
     """
 
     nvcc_path = build.get_nvcc_path()
+    hipcc_path = build.get_hipcc_path()
     summary = [
         '',
         '************************************************************',
@@ -324,12 +371,14 @@ def preconfigure_modules(compiler, settings):
         '  Library directories: {}'.format(str(settings['library_dirs'])),
         '  nvcc command       : {}'.format(
             nvcc_path if nvcc_path else '(not found)'),
+        '  hipcc command      : {}'.format(
+            hipcc_path if hipcc_path else '(not found)'),
         '',
         'Environment Variables:',
     ]
 
     for key in ['CFLAGS', 'LDFLAGS', 'LIBRARY_PATH',
-                'CUDA_PATH', 'NVTOOLSEXT_PATH', 'NVCC',
+                'CUDA_PATH', 'NVTOOLSEXT_PATH', 'NVCC', 'HIPCC',
                 'ROCM_HOME']:
         summary += ['  {:<16}: {}'.format(key, os.environ.get(key, '(none)'))]
 
@@ -345,16 +394,19 @@ def preconfigure_modules(compiler, settings):
         errmsg = []
 
         if module['name'] == 'cutensor':
-            cuda_version = build.get_cuda_version()
-            cuda_version = str(cuda_version // 1000) + '.' + \
-                str((cuda_version // 10) % 100)
             cutensor_path = os.environ.get('CUTENSOR_PATH', '')
             inc_path = os.path.join(cutensor_path, 'include')
             if os.path.exists(inc_path):
                 settings['include_dirs'].append(inc_path)
-            lib_path = os.path.join(cutensor_path, 'lib', cuda_version)
-            if os.path.exists(lib_path):
-                settings['library_dirs'].append(lib_path)
+            cuda_version = build.get_cuda_version()
+            cuda_major = str(cuda_version // 1000)
+            cuda_major_minor = cuda_major + '.' + \
+                str((cuda_version // 10) % 100)
+            for cuda_ver in (cuda_major_minor, cuda_major):
+                lib_path = os.path.join(cutensor_path, 'lib', cuda_ver)
+                if os.path.exists(lib_path):
+                    settings['library_dirs'].append(lib_path)
+                    break
 
         print('')
         print('-------- Configuring Module: {} --------'.format(
@@ -381,13 +433,11 @@ def preconfigure_modules(compiler, settings):
             # Fail on per-library condition check (version requirements etc.)
             installed = True
             errmsg = ['The library is installed but not supported.']
-        elif module['name'] == 'thrust' and nvcc_path is None:
+        elif (module['name'] in ('thrust', 'cub')
+                and (nvcc_path is None and hipcc_path is None)):
             installed = True
-            errmsg = ['nvcc command could not be found in PATH.',
-                      'Check your PATH environment variable.']
-        elif module['name'] == 'cub' and nvcc_path is None:
-            installed = True
-            errmsg = ['nvcc command could not be found in PATH.',
+            cmd = 'nvcc' if not use_hip else 'hipcc'
+            errmsg = ['{} command could not be found in PATH.'.format(cmd),
                       'Check your PATH environment variable.']
         else:
             installed = True
@@ -439,8 +489,6 @@ def preconfigure_modules(compiler, settings):
 def _rpath_base():
     if PLATFORM_LINUX:
         return '$ORIGIN'
-    elif PLATFORM_DARWIN:
-        return '@loader_path'
     else:
         raise Exception('not supported on this platform')
 
@@ -452,9 +500,6 @@ def make_extensions(options, compiler, use_cython):
     use_hip = not no_cuda and options['use_hip']
     settings = build.get_compiler_setting(use_hip)
 
-    if use_hip:
-        convert_modules_for_hip()
-
     include_dirs = settings['include_dirs']
 
     settings['include_dirs'] = [
@@ -497,7 +542,7 @@ def make_extensions(options, compiler, use_cython):
         if module['name'] not in available_modules:
             continue
 
-        s = settings.copy()
+        s = copy.deepcopy(settings)
         if not no_cuda:
             s['libraries'] = module['libraries']
 
@@ -510,50 +555,49 @@ def make_extensions(options, compiler, use_cython):
             # openmp is required for cusolver
             if use_hip:
                 pass
-            elif compiler.compiler_type == 'unix' and not PLATFORM_DARWIN:
-                # In mac environment, openmp is not required.
+            elif compiler.compiler_type == 'unix':
                 compile_args.append('-fopenmp')
                 link_args.append('-fopenmp')
             elif compiler.compiler_type == 'msvc':
                 compile_args.append('/openmp')
 
-        original_s = s
+        if module['name'] == 'dlpack':
+            # if any change is made to the DLPack header, we force recompiling
+            s['depends'] = ['./cupy/core/include/cupy/dlpack/dlpack.h']
+
         for f in module['file']:
-            s = copy.deepcopy(original_s)
+            s_file = copy.deepcopy(s)
             name = module_extension_name(f)
 
             rpath = []
             if not options['no_rpath']:
                 # Add library directories (e.g., `/usr/local/cuda/lib64`) to
                 # RPATH.
-                rpath += s['library_dirs']
+                rpath += s_file['library_dirs']
 
             if use_wheel_libs_rpath:
                 # Add `cupy/.data/lib` (where shared libraries included in
                 # wheels reside) to RPATH.
                 # The path is resolved relative to the module, e.g., use
-                # `$ORIGIN/.data/lib` for `cupy/cudnn.so` and
-                # `$ORIGIN/../.data/lib` for `cupy/cuda/cudnn.so`.
-                depth = name.count('.') - 1
+                # `$ORIGIN/../cupy/.data/lib` for `cupy/cudnn.so` and
+                # `$ORIGIN/../../../cupy/.data/lib` for
+                # `cupy_backends/cuda/libs/cudnn.so`.
+                depth = name.count('.')
                 rpath.append(
-                    '{}{}/.data/lib'.format(_rpath_base(), '/..' * depth))
+                    '{}{}/cupy/.data/lib'.format(_rpath_base(), '/..' * depth))
 
             if not PLATFORM_WIN32 and not PLATFORM_LINUX:
-                s['runtime_library_dirs'] = rpath
-            if (PLATFORM_LINUX and s['library_dirs']) or PLATFORM_DARWIN:
+                assert False, "macOS is no longer supported"
+            if (PLATFORM_LINUX and s_file['library_dirs']):
                 ldflag = '-Wl,'
                 if PLATFORM_LINUX:
                     ldflag += '--disable-new-dtags,'
                 ldflag += ','.join('-rpath,' + p for p in rpath)
-                args = s.setdefault('extra_link_args', [])
+                args = s_file.setdefault('extra_link_args', [])
                 args.append(ldflag)
-                if PLATFORM_DARWIN:
-                    # -rpath is only supported when targeting Mac OS X 10.5 or
-                    # later
-                    args.append('-mmacosx-version-min=10.5')
 
             sources = module_extension_sources(f, use_cython, no_cuda)
-            extension = setuptools.Extension(name, sources, **s)
+            extension = setuptools.Extension(name, sources, **s_file)
             ret.append(extension)
 
     return ret
@@ -580,6 +624,9 @@ def parse_args():
              'The former part is a full path of the source include file and '
              'the latter is the relative path within cupy wheel. '
              '(can be specified for multiple times)')
+    parser.add_argument(
+        '--cupy-wheel-metadata', type=str, default=None,
+        help='wheel metadata (cupy/.data/_wheel.json)')
     parser.add_argument(
         '--cupy-no-rpath', action='store_true', default=False,
         help='disable adding default library directories to RPATH')
@@ -603,6 +650,7 @@ def parse_args():
         'long_description': opts.cupy_long_description,
         'wheel_libs': opts.cupy_wheel_lib,  # list
         'wheel_includes': opts.cupy_wheel_include,  # list
+        'wheel_metadata': opts.cupy_wheel_metadata,
         'no_rpath': opts.cupy_no_rpath,
         'profile': opts.cupy_profile,
         'linetrace': opts.cupy_coverage,
@@ -634,78 +682,63 @@ def get_long_description():
 def prepare_wheel_libs():
     """Prepare shared libraries and include files for wheels.
 
-    On Windows, DLLs will be placed under `cupy/cuda`.
-    On other platforms, shared libraries are placed under `cupy/.data/lib` and
-    RUNPATH will be set to this directory later.
+    Shared libraries are placed under `cupy/.data/lib` and
+    RUNPATH will be set to this directory later (Linux only).
     Include files are placed under `cupy/.data/include`.
+
+    Returns the list of files (path relative to `cupy` module) to add to
+    the sdist/wheel distribution.
     """
-    data_dir = '.data'
+    data_dir = os.path.abspath(os.path.join('cupy', '.data'))
     if os.path.exists(data_dir):
         print('Removing directory: {}'.format(data_dir))
         shutil.rmtree(data_dir)
 
-    if PLATFORM_WIN32:
-        lib_dirname = 'cuda'
-        # Clean up existing libraries.
-        libfiles = glob.glob('cupy/{}/*.dll'.format(lib_dirname))
-        for libfile in libfiles:
-            print('Removing file: {}'.format(libfile))
-            os.remove(libfile)
-    else:
-        lib_dirname = os.path.join(data_dir, 'lib')
-
-    include_dirname = os.path.join(data_dir, 'include')
-
-    # Collect files to copy
+    # Generate list files to copy
+    # tuple of (src_path, dst_path)
     files_to_copy = []
 
     # Library files
-    lib_base_path = os.path.join('cupy', lib_dirname)
     for srcpath in cupy_setup_options['wheel_libs']:
         relpath = os.path.basename(srcpath)
-        dstpath = path.join(lib_base_path, relpath)
-        files_to_copy.append((
-            srcpath,
-            dstpath,
-            path.join(lib_dirname, relpath)))
+        dstpath = os.path.join(data_dir, 'lib', relpath)
+        files_to_copy.append((srcpath, dstpath))
 
     # Include files
-    include_base_path = os.path.join('cupy', include_dirname)
     for include_path_spec in cupy_setup_options['wheel_includes']:
-        # TODO(niboshi): Consider using platform-dependent path delimiter.
         srcpath, relpath = include_path_spec.rsplit(':', 1)
-        dstpath = os.path.join(include_base_path, relpath)
-        files_to_copy.append((
-            srcpath,
-            dstpath,
-            path.join(include_dirname, relpath)))
+        dstpath = os.path.join(data_dir, 'include', relpath)
+        files_to_copy.append((srcpath, dstpath))
+
+    # Wheel meta data
+    wheel_metadata = cupy_setup_options['wheel_metadata']
+    if wheel_metadata:
+        files_to_copy.append(
+            (wheel_metadata, os.path.join(data_dir, '_wheel.json')))
 
     # Copy
-    package_data = []
-    for srcpath, dstpath, package_path in files_to_copy:
+    for srcpath, dstpath in files_to_copy:
         # Note: symlink is resolved by shutil.copy2.
         print('Copying file for wheel: {}'.format(srcpath))
         dirpath = os.path.dirname(dstpath)
         if not os.path.isdir(dirpath):
             os.makedirs(dirpath)
         shutil.copy2(srcpath, dstpath)
-        package_data.append(package_path)
 
-    return package_data
-
-
-try:
-    import Cython
-    import Cython.Build
-    cython_version = pkg_resources.parse_version(Cython.__version__)
-    cython_available = (
-        cython_version >= required_cython_version and
-        cython_version not in ignore_cython_versions)
-except ImportError:
-    cython_available = False
+    return [os.path.relpath(x[1], 'cupy') for x in files_to_copy]
 
 
 def cythonize(extensions, arg_options):
+    # Delay importing Cython as it may be installed via setup_requires if
+    # the user does not have Cython installed.
+    import Cython
+    import Cython.Build
+    cython_version = pkg_resources.parse_version(Cython.__version__)
+    if (cython_version < required_cython_version or
+            cython_version in ignore_cython_versions):
+        raise AssertionError(
+            'Unsupported Cython version: {}'.format(cython_version))
+
     directive_keys = ('linetrace', 'profile')
     directives = {key: arg_options[key] for key in directive_keys}
 
@@ -716,6 +749,18 @@ def cythonize(extensions, arg_options):
     cythonize_options = {key: arg_options[key]
                          for key in cythonize_option_keys}
 
+    # Compile-time constants to be used in Cython code
+    compile_time_env = cythonize_options.get('compile_time_env')
+    if compile_time_env is None:
+        compile_time_env = {}
+        cythonize_options['compile_time_env'] = compile_time_env
+    compile_time_env['use_hip'] = arg_options['use_hip']
+    compile_time_env['cython_version'] = str(cython_version)
+    if use_hip or arg_options['no_cuda']:
+        compile_time_env['CUDA_VERSION'] = 0
+    else:
+        compile_time_env['CUDA_VERSION'] = build.get_cuda_version()
+
     return Cython.Build.cythonize(
         extensions, verbose=True, language_level=3,
         compiler_directives=directives, **cythonize_options)
@@ -754,7 +799,8 @@ def _nvcc_gencode_options(cuda_version):
 
     envcfg = os.getenv('CUPY_NVCC_GENERATE_CODE', None)
     if envcfg:
-        return ['--generate-code={}'.format(envcfg)]
+        return ['--generate-code={}'.format(arch)
+                for arch in envcfg.split(';') if len(arch) > 0]
 
     # The arch_list specifies virtual architectures, such as 'compute_61', and
     # real architectures, such as 'sm_61', for which the CUDA input files are
@@ -782,25 +828,39 @@ def _nvcc_gencode_options(cuda_version):
     # arch_list has an entry of ('compute_61', 'sm_61').
     #
     #     arch_list = [('compute_61', 'sm_61')]
+    #
+    # See the documentation of each CUDA version for the list of supported
+    # architectures:
+    #
+    #   https://docs.nvidia.com/cuda/cuda-compiler-driver-nvcc/index.html#options-for-steering-gpu-code-generation
 
-    arch_list = ['compute_30',
-                 'compute_50']
-
-    if cuda_version >= 10000:
-        arch_list += [('compute_60', 'sm_60'),
-                      ('compute_61', 'sm_61'),
-                      ('compute_70', 'sm_70'),
-                      ('compute_75', 'sm_75'),
-                      'compute_70']
+    if cuda_version >= 11000:
+        arch_list = ['compute_35',
+                     'compute_50',
+                     ('compute_60', 'sm_60'),
+                     ('compute_61', 'sm_61'),
+                     ('compute_70', 'sm_70'),
+                     ('compute_75', 'sm_75'),
+                     ('compute_80', 'sm_80'),
+                     'compute_80']
+    elif cuda_version >= 10000:
+        arch_list = ['compute_30',
+                     'compute_50',
+                     ('compute_60', 'sm_60'),
+                     ('compute_61', 'sm_61'),
+                     ('compute_70', 'sm_70'),
+                     ('compute_75', 'sm_75'),
+                     'compute_70']
     elif cuda_version >= 9000:
-        arch_list += [('compute_60', 'sm_60'),
-                      ('compute_61', 'sm_61'),
-                      ('compute_70', 'sm_70'),
-                      'compute_70']
-    elif cuda_version >= 8000:
-        arch_list += [('compute_60', 'sm_60'),
-                      ('compute_61', 'sm_61'),
-                      'compute_60']
+        arch_list = ['compute_30',
+                     'compute_50',
+                     ('compute_60', 'sm_60'),
+                     ('compute_61', 'sm_61'),
+                     ('compute_70', 'sm_70'),
+                     'compute_70']
+    else:
+        # This should not happen.
+        assert False
 
     options = []
     for arch in arch_list:
@@ -826,42 +886,44 @@ class _UnixCCompiler(unixccompiler.UnixCCompiler):
                 self, obj, src, ext, cc_args, extra_postargs, pp_opts)
 
         if use_hip:
-            return self._comiple_unix_hipcc(
+            return self._compile_unix_hipcc(
                 obj, src, ext, cc_args, extra_postargs, pp_opts)
 
         # For CUDA C source files, compile them with NVCC.
-        _compiler_so = self.compiler_so
         try:
             nvcc_path = build.get_nvcc_path()
-            base_opts = build.get_compiler_base_options()
-            self.set_executable('compiler_so', nvcc_path)
+            base_opts = build.get_compiler_base_options(nvcc_path)
+            compiler_so = nvcc_path
 
             cuda_version = build.get_cuda_version()
             postargs = _nvcc_gencode_options(cuda_version) + [
-                '-O2', '--compiler-options="-fPIC"', '--std=c++11']
+                '-O2', '--compiler-options="-fPIC"']
+            if cuda_version >= 11020:
+                postargs += ['--std=c++14']
+            else:
+                postargs += ['--std=c++11']
             print('NVCC options:', postargs)
 
-            return unixccompiler.UnixCCompiler._compile(
-                self, obj, src, ext, base_opts + cc_args, postargs, pp_opts)
-        finally:
-            self.compiler_so = _compiler_so
+            self.spawn(compiler_so + base_opts + cc_args + [src, '-o', obj] +
+                       postargs)
+        except errors.DistutilsExecError as e:
+            raise errors.CompileError(str(e))
 
-    def _comiple_unix_hipcc(self,
+    def _compile_unix_hipcc(self,
                             obj, src, ext, cc_args, extra_postargs, pp_opts):
         # For CUDA C source files, compile them with HIPCC.
-        _compiler_so = self.compiler_so
         try:
-            rcom_path = build.get_hipcc_path()
-            base_opts = build.get_compiler_base_options()
-            self.set_executable('compiler_so', rcom_path)
+            rocm_path = build.get_hipcc_path()
+            base_opts = build.get_compiler_base_options(rocm_path)
+            compiler_so = rocm_path
 
-            postargs = ['-O2', '-fPIC']
+            postargs = ['-O2', '-fPIC', '--include', 'hip_runtime.h']
             print('HIPCC options:', postargs)
 
-            return unixccompiler.UnixCCompiler._compile(
-                self, obj, src, ext, base_opts + cc_args, postargs, pp_opts)
-        finally:
-            self.compiler_so = _compiler_so
+            self.spawn(compiler_so + base_opts + cc_args + [src, '-o', obj] +
+                       postargs)
+        except errors.DistutilsExecError as e:
+            raise errors.CompileError(str(e))
 
     def link(self, target_desc, objects, output_filename, *args):
         use_hipcc = False
@@ -894,7 +956,6 @@ class _MSVCCompiler(msvccompiler.MSVCCompiler):
                     include_dirs=None, debug=0, extra_preargs=None,
                     extra_postargs=None, depends=None):
         # Compile CUDA C files, mainly derived from UnixCCompiler._compile().
-
         macros, objects, extra_postargs, pp_opts, _build = \
             self._setup_compile(output_dir, macros, include_dirs, sources,
                                 depends, extra_postargs)
@@ -903,8 +964,15 @@ class _MSVCCompiler(msvccompiler.MSVCCompiler):
         cc_args = self._get_cc_args(pp_opts, debug, extra_preargs)
         cuda_version = build.get_cuda_version()
         postargs = _nvcc_gencode_options(cuda_version) + ['-O2']
+        if cuda_version >= 11020:
+            # MSVC 14.0 (2015) is deprecated for CUDA 11.2 but we need it
+            # to build CuPy because some Python versions were built using it.
+            # REF: https://wiki.python.org/moin/WindowsCompilers
+            postargs += ['-allow-unsupported-compiler']
         postargs += ['-Xcompiler', '/MD']
-        postargs += ['-Xcompiler', '/bigobj']
+        # This is to compile thrust with MSVC2015
+        if cuda_version >= 11020:
+            postargs += ['--std=c++14']
         print('NVCC options:', postargs)
 
         for obj in objects:
@@ -921,6 +989,9 @@ class _MSVCCompiler(msvccompiler.MSVCCompiler):
 
     def compile(self, sources, **kwargs):
         # Split CUDA C sources and others.
+        if use_hip:
+            raise RuntimeError('ROCm is not supported on Windows')
+
         cu_sources = []
         other_sources = []
         for source in sources:
@@ -940,24 +1011,13 @@ class _MSVCCompiler(msvccompiler.MSVCCompiler):
         return other_objects + cu_objects
 
 
-class sdist_with_cython(sdist.sdist):
-
-    """Custom `sdist` command with cyhonizing."""
-
-    def __init__(self, *args, **kwargs):
-        if not cython_available:
-            raise RuntimeError('Cython is required to make sdist.')
-        ext_modules = get_ext_modules(True)  # get .pyx modules
-        cythonize(ext_modules, cupy_setup_options)
-        sdist.sdist.__init__(self, *args, **kwargs)
-
-
 class custom_build_ext(build_ext.build_ext):
 
     """Custom `build_ext` command to include CUDA C source files."""
 
     def run(self):
-        if build.get_nvcc_path() is not None:
+        if (build.get_nvcc_path() is not None
+                or build.get_hipcc_path() is not None):
             def wrap_new_compiler(func):
                 def _wrap_new_compiler(*args, **kwargs):
                     try:
@@ -974,8 +1034,13 @@ class custom_build_ext(build_ext.build_ext):
             # Intentionally causes DistutilsPlatformError in
             # ccompiler.new_compiler() function to hook.
             self.compiler = 'nvidia'
-        if cython_available:
-            ext_modules = get_ext_modules(True)  # get .pyx modules
-            cythonize(ext_modules, cupy_setup_options)
+        ext_modules = get_ext_modules(True)  # get .pyx modules
+        cythonize(ext_modules, cupy_setup_options)
         check_extensions(self.extensions)
         build_ext.build_ext.run(self)
+
+    def build_extensions(self):
+        num_jobs = int(os.environ.get('CUPY_NUM_BUILD_JOBS', '1'))
+        if num_jobs > 1:
+            self.parallel = num_jobs
+        super().build_extensions()
