@@ -46,7 +46,6 @@ class TestFromData(unittest.TestCase):
 
     @testing.for_orders('CFAK')
     @testing.for_all_dtypes()
-    @testing.with_requires('numpy>=1.10')
     @testing.numpy_cupy_array_equal()
     def test_array_from_numpy_broad_cast(self, xp, dtype, order):
         a = testing.shaped_arange((2, 1, 4), numpy, dtype)
@@ -156,6 +155,15 @@ class TestFromData(unittest.TestCase):
         a = [xp.array(i, dtype=dtype) for i in range(2)]
         return xp.array(a, order=order)
 
+    @testing.for_orders('CFAK')
+    @testing.for_all_dtypes()
+    @testing.numpy_cupy_array_equal()
+    def test_array_from_nested_list_of_cupy_scalar(self, xp, dtype, order):
+        # compares numpy.array(<list of numpy.ndarray>) with
+        # cupy.array(<list of cupy.ndarray>)
+        a = [[xp.array(i, dtype=dtype)] for i in range(2)]
+        return xp.array(a, order=order)
+
     @testing.for_orders('CFAK')
     @testing.for_all_dtypes()
     @testing.numpy_cupy_array_equal()
@@ -450,7 +458,7 @@ class TestFromData(unittest.TestCase):
             return xp.fromfile(fh, dtype="u1")
 
 
-max_cuda_array_interface_version = 2
+max_cuda_array_interface_version = 3
 
 
 @testing.gpu
@@ -501,6 +509,14 @@ class TestCudaArrayInterface(unittest.TestCase):
         assert a.data.ptr == 0
         assert a.size == 0
 
+    @testing.for_all_dtypes()
+    def test_asnumpy(self, dtype):
+        a = testing.shaped_arange((2, 3, 4), cupy, dtype)
+        b = DummyObjectWithCudaArrayInterface(a, self.ver, self.strides)
+        a_cpu = cupy.asnumpy(a)
+        b_cpu = cupy.asnumpy(b)
+        testing.assert_array_equal(a_cpu, b_cpu)
+
 
 @testing.gpu
 @testing.parameterize(*testing.product({
@@ -519,13 +535,32 @@ class TestCudaArrayInterfaceMaskedArray(unittest.TestCase):
         assert 'does not support' in str(ex.value)
 
 
+@testing.slow
+@testing.gpu
+class TestCudaArrayInterfaceBigArray(unittest.TestCase):
+    def test_with_over_size_array(self):
+        # real example from #3009
+        size = 5 * 10**8
+        try:
+            a = testing.shaped_random((size,), cupy, cupy.float64)
+            b = cupy.asarray(DummyObjectWithCudaArrayInterface(a, 2, None))
+            testing.assert_array_equal(a, b)
+        except cupy.cuda.memory.OutOfMemoryError:
+            pass
+        else:
+            del b, a
+        finally:
+            cupy.get_default_memory_pool().free_all_blocks()
+
+
 class DummyObjectWithCudaArrayInterface(object):
-    def __init__(self, a, ver, include_strides=False, mask=None):
+    def __init__(self, a, ver, include_strides=False, mask=None, stream=None):
         assert ver in tuple(range(max_cuda_array_interface_version+1))
         self.a = a
         self.ver = ver
         self.include_strides = include_strides
         self.mask = mask
+        self.stream = stream
 
     @property
     def __cuda_array_interface__(self):
@@ -547,6 +582,15 @@ class DummyObjectWithCudaArrayInterface(object):
             desc['strides'] = self.a.strides
         if self.mask is not None:
             desc['mask'] = self.mask
+        # The stream field is kept here for compliance. However, since the
+        # synchronization is done via calling a cpdef function, which cannot
+        # be mock-tested.
+        if self.stream is not None:
+            # TODO(leofang): how about PTDS?
+            if self.stream is cuda.Stream.null:
+                desc['stream'] = 1  # TODO(leofang): use runtime.streamLegacy
+            else:
+                desc['stream'] = self.stream.ptr
         return desc
 
 
