@@ -1,9 +1,11 @@
 import unittest
 
 import numpy
-import six
+import pytest
 
 import cupy
+import cupy.core._accelerator as _acc
+from cupy.core import _cub_reduction
 from cupy import testing
 
 
@@ -58,7 +60,6 @@ class TestSumprod(unittest.TestCase):
         return a.sum(axis=1)
 
     @testing.slow
-    @testing.with_requires('numpy>=1.10')
     @testing.numpy_cupy_allclose()
     def test_sum_axis_huge(self, xp):
         a = testing.shaped_random((2048, 1, 1024), xp, 'b')
@@ -124,11 +125,17 @@ class TestSumprod(unittest.TestCase):
         a = testing.shaped_arange((20, 30, 40, 50), xp, dtype)
         return a.sum(axis=(0, 2, 3))
 
+    @testing.for_all_dtypes()
+    @testing.numpy_cupy_allclose()
+    def test_sum_empty_axis(self, xp, dtype):
+        a = testing.shaped_arange((2, 3, 4, 5), xp, dtype)
+        return a.sum(axis=())
+
     @testing.for_all_dtypes_combination(names=['src_dtype', 'dst_dtype'])
     @testing.numpy_cupy_allclose()
     def test_sum_dtype(self, xp, src_dtype, dst_dtype):
         if not xp.can_cast(src_dtype, dst_dtype):
-            return xp.array([])  # skip
+            pytest.skip()
         a = testing.shaped_arange((2, 3, 4), xp, src_dtype)
         return a.sum(dtype=dst_dtype)
 
@@ -136,7 +143,7 @@ class TestSumprod(unittest.TestCase):
     @testing.numpy_cupy_allclose()
     def test_sum_keepdims_and_dtype(self, xp, src_dtype, dst_dtype):
         if not xp.can_cast(src_dtype, dst_dtype):
-            return xp.array([])  # skip
+            pytest.skip()
         a = testing.shaped_arange((2, 3, 4), xp, src_dtype)
         return a.sum(axis=2, dtype=dst_dtype, keepdims=True)
 
@@ -188,11 +195,240 @@ class TestSumprod(unittest.TestCase):
     @testing.numpy_cupy_allclose()
     def test_prod_dtype(self, xp, src_dtype, dst_dtype):
         if not xp.can_cast(src_dtype, dst_dtype):
-            return xp.array([])  # skip
+            pytest.skip()
         a = testing.shaped_arange((2, 3), xp, src_dtype)
         return a.prod(dtype=dst_dtype)
 
 
+# This class compares CUB results against NumPy's
+@testing.parameterize(*testing.product({
+    'shape': [(10,), (10, 20), (10, 20, 30), (10, 20, 30, 40)],
+    'order': ('C', 'F'),
+    'backend': ('device', 'block'),
+}))
+@testing.gpu
+@unittest.skipUnless(cupy.cuda.cub.available, 'The CUB routine is not enabled')
+class TestCubReduction(unittest.TestCase):
+
+    def setUp(self):
+        self.old_routine_accelerators = _acc.get_routine_accelerators()
+        self.old_reduction_accelerators = _acc.get_reduction_accelerators()
+        if self.backend == 'device':
+            _acc.set_routine_accelerators(['cub'])
+            _acc.set_reduction_accelerators([])
+        elif self.backend == 'block':
+            _acc.set_routine_accelerators([])
+            _acc.set_reduction_accelerators(['cub'])
+
+    def tearDown(self):
+        _acc.set_routine_accelerators(self.old_routine_accelerators)
+        _acc.set_reduction_accelerators(self.old_reduction_accelerators)
+
+    @testing.for_contiguous_axes()
+    # sum supports less dtypes; don't test float16 as it's not as accurate?
+    @testing.for_dtypes('lLfdFD')
+    @testing.numpy_cupy_allclose(rtol=1E-5)
+    def test_cub_sum(self, xp, dtype, axis):
+        a = testing.shaped_random(self.shape, xp, dtype)
+        if self.order in ('c', 'C'):
+            a = xp.ascontiguousarray(a)
+        elif self.order in ('f', 'F'):
+            a = xp.asfortranarray(a)
+
+        if xp is numpy:
+            return a.sum(axis=axis)
+
+        # xp is cupy, first ensure we really use CUB
+        ret = cupy.empty(())  # Cython checks return type, need to fool it
+        if self.backend == 'device':
+            func_name = 'cupy.core._routines_math.cub.'
+            if len(axis) == len(self.shape):
+                func_name += 'device_reduce'
+            else:
+                func_name += 'device_segmented_reduce'
+            with testing.AssertFunctionIsCalled(func_name, return_value=ret):
+                a.sum(axis=axis)
+        elif self.backend == 'block':
+            # this is the only function we can mock; the rest is cdef'd
+            func_name = 'cupy.core._cub_reduction.'
+            func_name += '_SimpleCubReductionKernel_get_cached_function'
+            func = _cub_reduction._SimpleCubReductionKernel_get_cached_function
+            if len(axis) == len(self.shape):
+                times_called = 2  # two passes
+            else:
+                times_called = 1  # one pass
+            with testing.AssertFunctionIsCalled(
+                    func_name, wraps=func, times_called=times_called):
+                a.sum(axis=axis)
+        # ...then perform the actual computation
+        return a.sum(axis=axis)
+
+    # sum supports less dtypes; don't test float16 as it's not as accurate?
+    @testing.for_dtypes('lLfdFD')
+    @testing.numpy_cupy_allclose(rtol=1E-5, contiguous_check=False)
+    def test_cub_sum_empty_axis(self, xp, dtype):
+        a = testing.shaped_random(self.shape, xp, dtype)
+        if self.order in ('c', 'C'):
+            a = xp.ascontiguousarray(a)
+        elif self.order in ('f', 'F'):
+            a = xp.asfortranarray(a)
+        return a.sum(axis=())
+
+    @testing.for_contiguous_axes()
+    # prod supports less dtypes; don't test float16 as it's not as accurate?
+    @testing.for_dtypes('lLfdFD')
+    @testing.numpy_cupy_allclose(rtol=1E-5)
+    def test_cub_prod(self, xp, dtype, axis):
+        a = testing.shaped_random(self.shape, xp, dtype)
+        if self.order in ('c', 'C'):
+            a = xp.ascontiguousarray(a)
+        elif self.order in ('f', 'F'):
+            a = xp.asfortranarray(a)
+
+        if xp is numpy:
+            return a.prod(axis=axis)
+
+        # xp is cupy, first ensure we really use CUB
+        ret = cupy.empty(())  # Cython checks return type, need to fool it
+        if self.backend == 'device':
+            func_name = 'cupy.core._routines_math.cub.'
+            if len(axis) == len(self.shape):
+                func_name += 'device_reduce'
+            else:
+                func_name += 'device_segmented_reduce'
+            with testing.AssertFunctionIsCalled(func_name, return_value=ret):
+                a.prod(axis=axis)
+        elif self.backend == 'block':
+            # this is the only function we can mock; the rest is cdef'd
+            func_name = 'cupy.core._cub_reduction.'
+            func_name += '_SimpleCubReductionKernel_get_cached_function'
+            func = _cub_reduction._SimpleCubReductionKernel_get_cached_function
+            if len(axis) == len(self.shape):
+                times_called = 2  # two passes
+            else:
+                times_called = 1  # one pass
+            with testing.AssertFunctionIsCalled(
+                    func_name, wraps=func, times_called=times_called):
+                a.prod(axis=axis)
+        # ...then perform the actual computation
+        return a.prod(axis=axis)
+
+    # TODO(leofang): test axis after support is added
+    # don't test float16 as it's not as accurate?
+    @testing.for_dtypes('bhilBHILfdF')
+    @testing.numpy_cupy_allclose(rtol=1E-4)
+    def test_cub_cumsum(self, xp, dtype):
+        if self.backend == 'block':
+            raise unittest.SkipTest('does not support')
+
+        a = testing.shaped_random(self.shape, xp, dtype)
+        if self.order in ('c', 'C'):
+            a = xp.ascontiguousarray(a)
+        elif self.order in ('f', 'F'):
+            a = xp.asfortranarray(a)
+
+        if xp is numpy:
+            return a.cumsum()
+
+        # xp is cupy, first ensure we really use CUB
+        ret = cupy.empty(())  # Cython checks return type, need to fool it
+        func = 'cupy.core._routines_math.cub.device_scan'
+        with testing.AssertFunctionIsCalled(func, return_value=ret):
+            a.cumsum()
+        # ...then perform the actual computation
+        return a.cumsum()
+
+    # TODO(leofang): test axis after support is added
+    # don't test float16 as it's not as accurate?
+    @testing.for_dtypes('bhilBHILfdF')
+    @testing.numpy_cupy_allclose(rtol=1E-4)
+    def test_cub_cumprod(self, xp, dtype):
+        if self.backend == 'block':
+            raise unittest.SkipTest('does not support')
+
+        a = testing.shaped_random(self.shape, xp, dtype)
+        if self.order in ('c', 'C'):
+            a = xp.ascontiguousarray(a)
+        elif self.order in ('f', 'F'):
+            a = xp.asfortranarray(a)
+
+        if xp is numpy:
+            result = a.cumprod()
+            return self._mitigate_cumprod(xp, dtype, result)
+
+        # xp is cupy, first ensure we really use CUB
+        ret = cupy.empty(())  # Cython checks return type, need to fool it
+        func = 'cupy.core._routines_math.cub.device_scan'
+        with testing.AssertFunctionIsCalled(func, return_value=ret):
+            a.cumprod()
+        # ...then perform the actual computation
+        result = a.cumprod()
+        return self._mitigate_cumprod(xp, dtype, result)
+
+    def _mitigate_cumprod(self, xp, dtype, result):
+        # for testing cumprod against complex arrays, the gotcha is CuPy may
+        # produce only Inf at the position where NumPy starts to give NaN. So,
+        # an error would be raised during assert_allclose where the positions
+        # of NaNs are examined. Since this is both algorithm and architecture
+        # dependent, we have no control over this behavior and can only
+        # circumvent the issue by manually converting Inf to NaN
+        if dtype in (numpy.complex64, numpy.complex128):
+            pos = xp.where(xp.isinf(result))
+            result[pos] = xp.nan + 1j * xp.nan
+        return result
+
+
+# This class compares cuTENSOR results against NumPy's
+@testing.parameterize(*testing.product({
+    'shape': [(10,), (10, 20), (10, 20, 30), (10, 20, 30, 40)],
+    'order': ('C', 'F'),
+}))
+@testing.gpu
+@unittest.skipUnless(cupy.cuda.cutensor.available,
+                     'The cuTENSOR routine is not enabled')
+class TestCuTensorReduction(unittest.TestCase):
+
+    def setUp(self):
+        self.old_accelerators = cupy.core.get_routine_accelerators()
+        cupy.core.set_routine_accelerators(['cutensor'])
+
+    def tearDown(self):
+        cupy.core.set_routine_accelerators(self.old_accelerators)
+
+    @testing.for_contiguous_axes()
+    # sum supports less dtypes; don't test float16 as it's not as accurate?
+    @testing.for_dtypes('lLfdFD')
+    @testing.numpy_cupy_allclose(rtol=1E-5, contiguous_check=False)
+    def test_cutensor_sum(self, xp, dtype, axis):
+        a = testing.shaped_random(self.shape, xp, dtype)
+        if self.order in ('c', 'C'):
+            a = xp.ascontiguousarray(a)
+        elif self.order in ('f', 'F'):
+            a = xp.asfortranarray(a)
+
+        if xp is numpy:
+            return a.sum(axis=axis)
+
+        # xp is cupy, first ensure we really use cuTENSOR
+        ret = cupy.empty(())  # Cython checks return type, need to fool it
+        func = 'cupy.cutensor._try_reduction_routine'
+        with testing.AssertFunctionIsCalled(func, return_value=ret):
+            a.sum(axis=axis)
+        # ...then perform the actual computation
+        return a.sum(axis=axis)
+
+    # sum supports less dtypes; don't test float16 as it's not as accurate?
+    @testing.for_dtypes('lLfdFD')
+    @testing.numpy_cupy_allclose(rtol=1E-5, contiguous_check=False)
+    def test_cutensor_sum_empty_axis(self, xp, dtype):
+        a = testing.shaped_random(self.shape, xp, dtype)
+        if self.order in ('c', 'C'):
+            a = xp.ascontiguousarray(a)
+        elif self.order in ('f', 'F'):
+            a = xp.asfortranarray(a)
+        return a.sum(axis=())
+
+
 @testing.parameterize(
     *testing.product({
         'shape': [(2, 3, 4), (20, 30, 40)],
@@ -303,14 +539,12 @@ class TestNansumNanprodHuge(unittest.TestCase):
         return xp.nansum(a, axis=2)
 
     @testing.slow
-    @testing.with_requires('numpy>=1.10')
     @testing.numpy_cupy_allclose(atol=1e-1)
     def test_nansum_axis_huge(self, xp):
         return self._test(
             xp, (slice(None, None), slice(None, None), slice(1, 2)))
 
     @testing.slow
-    @testing.with_requires('numpy>=1.10')
     @testing.numpy_cupy_allclose(atol=1e-2)
     def test_nansum_axis_huge_halfnan(self, xp):
         return self._test(
@@ -330,6 +564,22 @@ class TestCumsum(unittest.TestCase):
         a = testing.shaped_arange((5,), xp, dtype)
         return xp.cumsum(a)
 
+    @testing.for_all_dtypes()
+    @testing.numpy_cupy_allclose()
+    def test_cumsum_out(self, xp, dtype):
+        a = testing.shaped_arange((5,), xp, dtype)
+        out = xp.zeros((5,), dtype=dtype)
+        xp.cumsum(a, out=out)
+        return out
+
+    @testing.for_all_dtypes()
+    @testing.numpy_cupy_allclose()
+    def test_cumsum_out_noncontiguous(self, xp, dtype):
+        a = testing.shaped_arange((5,), xp, dtype)
+        out = xp.zeros((10,), dtype=dtype)[::2]  # Non contiguous view
+        xp.cumsum(a, out=out)
+        return out
+
     @testing.for_all_dtypes()
     @testing.numpy_cupy_allclose()
     def test_cumsum_2dim(self, xp, dtype):
@@ -340,58 +590,78 @@ class TestCumsum(unittest.TestCase):
     @testing.numpy_cupy_allclose(contiguous_check=False)
     def test_cumsum_axis(self, xp, dtype):
         n = len(axes)
-        a = testing.shaped_arange(tuple(six.moves.range(4, 4 + n)), xp, dtype)
+        a = testing.shaped_arange(tuple(range(4, 4 + n)), xp, dtype)
         return xp.cumsum(a, axis=self.axis)
 
+    @testing.for_all_dtypes()
+    @testing.numpy_cupy_allclose()
+    def test_cumsum_axis_out(self, xp, dtype):
+        n = len(axes)
+        shape = tuple(range(4, 4 + n))
+        a = testing.shaped_arange(shape, xp, dtype)
+        out = xp.zeros(shape, dtype=dtype)
+        xp.cumsum(a, axis=self.axis, out=out)
+        return out
+
+    @testing.for_all_dtypes()
+    @testing.numpy_cupy_allclose()
+    def test_cumsum_axis_out_noncontiguous(self, xp, dtype):
+        n = len(axes)
+        shape = tuple(range(4, 4 + n))
+        a = testing.shaped_arange(shape, xp, dtype)
+        out = xp.zeros((8,)+shape[1:], dtype=dtype)[::2]  # Non contiguous view
+        xp.cumsum(a, axis=self.axis, out=out)
+        return out
+
     @testing.for_all_dtypes()
     @testing.numpy_cupy_allclose(contiguous_check=False)
     def test_ndarray_cumsum_axis(self, xp, dtype):
         n = len(axes)
-        a = testing.shaped_arange(tuple(six.moves.range(4, 4 + n)), xp, dtype)
+        a = testing.shaped_arange(tuple(range(4, 4 + n)), xp, dtype)
         return a.cumsum(axis=self.axis)
 
     @testing.for_all_dtypes()
     @testing.numpy_cupy_allclose()
     def test_cumsum_axis_empty(self, xp, dtype):
         n = len(axes)
-        a = testing.shaped_arange(tuple(six.moves.range(0, n)), xp, dtype)
+        a = testing.shaped_arange(tuple(range(0, n)), xp, dtype)
         return xp.cumsum(a, axis=self.axis)
 
     @testing.for_all_dtypes()
-    @testing.with_requires('numpy>=1.13')
-    @testing.numpy_cupy_raises()
-    def test_invalid_axis_lower1(self, xp, dtype):
-        a = testing.shaped_arange((4, 5), xp, dtype)
-        return xp.cumsum(a, axis=-a.ndim - 1)
+    def test_invalid_axis_lower1(self, dtype):
+        for xp in (numpy, cupy):
+            a = testing.shaped_arange((4, 5), xp, dtype)
+            with pytest.raises(numpy.AxisError):
+                xp.cumsum(a, axis=-a.ndim - 1)
 
     @testing.for_all_dtypes()
     def test_invalid_axis_lower2(self, dtype):
         a = testing.shaped_arange((4, 5), cupy, dtype)
-        with self.assertRaises(cupy.core._AxisError):
+        with self.assertRaises(numpy.AxisError):
             return cupy.cumsum(a, axis=-a.ndim - 1)
 
     @testing.for_all_dtypes()
-    @testing.with_requires('numpy>=1.13')
-    @testing.numpy_cupy_raises()
-    def test_invalid_axis_upper1(self, xp, dtype):
-        a = testing.shaped_arange((4, 5), xp, dtype)
-        return xp.cumsum(a, axis=a.ndim + 1)
+    def test_invalid_axis_upper1(self, dtype):
+        for xp in (numpy, cupy):
+            a = testing.shaped_arange((4, 5), xp, dtype)
+            with pytest.raises(numpy.AxisError):
+                xp.cumsum(a, axis=a.ndim + 1)
 
     @testing.for_all_dtypes()
     def test_invalid_axis_upper2(self, dtype):
         a = testing.shaped_arange((4, 5), cupy, dtype)
-        with self.assertRaises(cupy.core._AxisError):
+        with self.assertRaises(numpy.AxisError):
             return cupy.cumsum(a, axis=a.ndim + 1)
 
-    @testing.numpy_cupy_allclose()
-    def test_cumsum_arraylike(self, xp):
-        return xp.cumsum((1, 2, 3))
+    def test_cumsum_arraylike(self):
+        with self.assertRaises(TypeError):
+            return cupy.cumsum((1, 2, 3))
 
     @testing.for_float_dtypes()
-    @testing.numpy_cupy_allclose()
-    def test_cumsum_numpy_array(self, xp, dtype):
+    def test_cumsum_numpy_array(self, dtype):
         a_numpy = numpy.arange(8, dtype=dtype)
-        return xp.cumsum(a_numpy)
+        with self.assertRaises(TypeError):
+            return cupy.cumsum(a_numpy)
 
 
 @testing.gpu
@@ -403,6 +673,22 @@ class TestCumprod(unittest.TestCase):
         a = testing.shaped_arange((5,), xp, dtype)
         return xp.cumprod(a)
 
+    @testing.for_all_dtypes()
+    @testing.numpy_cupy_allclose()
+    def test_cumprod_out(self, xp, dtype):
+        a = testing.shaped_arange((5,), xp, dtype)
+        out = xp.zeros((5,), dtype=dtype)
+        xp.cumprod(a, out=out)
+        return out
+
+    @testing.for_all_dtypes()
+    @testing.numpy_cupy_allclose()
+    def test_cumprod_out_noncontiguous(self, xp, dtype):
+        a = testing.shaped_arange((5,), xp, dtype)
+        out = xp.zeros((10,), dtype=dtype)[::2]  # Non contiguous view
+        xp.cumprod(a, out=out)
+        return out
+
     @testing.for_all_dtypes()
     @testing.numpy_cupy_allclose(rtol=1e-6)
     def test_cumprod_2dim_without_axis(self, xp, dtype):
@@ -429,50 +715,50 @@ class TestCumprod(unittest.TestCase):
         a = cupy.ones(size, 'b')
         result = cupy.cumprod(a, dtype='b')
         del a
-        self.assertTrue((result == 1).all())
+        assert (result == 1).all()
         # Free huge memory for slow test
         del result
         cupy.get_default_memory_pool().free_all_blocks()
 
     @testing.for_all_dtypes()
-    @testing.with_requires('numpy>=1.13')
-    @testing.numpy_cupy_raises()
-    def test_invalid_axis_lower1(self, xp, dtype):
-        a = testing.shaped_arange((4, 5), xp, dtype)
-        return xp.cumprod(a, axis=-a.ndim - 1)
+    def test_invalid_axis_lower1(self, dtype):
+        for xp in (numpy, cupy):
+            a = testing.shaped_arange((4, 5), xp, dtype)
+            with pytest.raises(numpy.AxisError):
+                xp.cumprod(a, axis=-a.ndim - 1)
 
     @testing.for_all_dtypes()
     def test_invalid_axis_lower2(self, dtype):
-        a = testing.shaped_arange((4, 5), cupy, dtype)
-        with self.assertRaises(cupy.core._AxisError):
-            return cupy.cumprod(a, axis=-a.ndim - 1)
+        for xp in (numpy, cupy):
+            a = testing.shaped_arange((4, 5), xp, dtype)
+            with pytest.raises(numpy.AxisError):
+                xp.cumprod(a, axis=-a.ndim - 1)
 
     @testing.for_all_dtypes()
-    @testing.with_requires('numpy>=1.13')
-    @testing.numpy_cupy_raises()
-    def test_invalid_axis_upper1(self, xp, dtype):
-        a = testing.shaped_arange((4, 5), xp, dtype)
-        return xp.cumprod(a, axis=a.ndim)
+    def test_invalid_axis_upper1(self, dtype):
+        for xp in (numpy, cupy):
+            a = testing.shaped_arange((4, 5), xp, dtype)
+            with pytest.raises(numpy.AxisError):
+                return xp.cumprod(a, axis=a.ndim)
 
     @testing.for_all_dtypes()
     def test_invalid_axis_upper2(self, dtype):
         a = testing.shaped_arange((4, 5), cupy, dtype)
-        with self.assertRaises(cupy.core._AxisError):
+        with self.assertRaises(numpy.AxisError):
             return cupy.cumprod(a, axis=a.ndim)
 
-    @testing.numpy_cupy_allclose()
-    def test_cumprod_arraylike(self, xp):
-        return xp.cumprod((1, 2, 3))
+    def test_cumprod_arraylike(self):
+        with self.assertRaises(TypeError):
+            return cupy.cumprod((1, 2, 3))
 
     @testing.for_float_dtypes()
-    @testing.numpy_cupy_allclose()
-    def test_cumprod_numpy_array(self, xp, dtype):
+    def test_cumprod_numpy_array(self, dtype):
         a_numpy = numpy.arange(1, 6, dtype=dtype)
-        return xp.cumprod(a_numpy)
+        with self.assertRaises(TypeError):
+            return cupy.cumprod(a_numpy)
 
 
 @testing.gpu
-@testing.with_requires('numpy>=1.14')  # NumPy issue #9251
 class TestDiff(unittest.TestCase):
 
     @testing.for_all_dtypes()
@@ -527,3 +813,12 @@ class TestDiff(unittest.TestCase):
     def test_diff_2dim_with_scalar_append(self, xp, dtype):
         a = testing.shaped_arange((4, 5), xp, dtype)
         return xp.diff(a, prepend=1, append=0)
+
+    @testing.with_requires('numpy>=1.16')
+    def test_diff_invalid_axis(self):
+        for xp in (numpy, cupy):
+            a = testing.shaped_arange((2, 3, 4), xp)
+            with pytest.raises(numpy.AxisError):
+                xp.diff(a, axis=3)
+            with pytest.raises(numpy.AxisError):
+                xp.diff(a, axis=-4)
