@@ -4,12 +4,11 @@ import threading
 import unittest
 
 import numpy
-import six
+import pytest
 
 import cupy
-from cupy import core
 from cupy import cuda
-from cupy.random import generator
+from cupy.random import _generator
 from cupy import testing
 from cupy.testing import attr
 from cupy.testing import condition
@@ -38,8 +37,8 @@ def numpy_cupy_equal_continuous_distribution(significance_level, name='xp'):
             kw[name] = numpy
             numpy_result = impl(self, *args, **kw)
 
-            self.assertIsNotNone(cupy_result)
-            self.assertIsNotNone(numpy_result)
+            assert cupy_result is not None
+            assert numpy_result is not None
             d_plus, d_minus, p_value = \
                 two_sample_Kolmogorov_Smirnov_test(
                     cupy.asnumpy(cupy_result), numpy_result)
@@ -78,13 +77,21 @@ def two_sample_Kolmogorov_Smirnov_test(observed1, observed2):
     return d_plus, d_minus, p
 
 
+def _get_size(size):
+    # CuPy returns an ndarray of shape () even if size=None.
+    # cf. NumPy returns a Python scalar if size=None.
+    if size is None:
+        return ()
+    return cupy.core.get_size(size)
+
+
 class RandomGeneratorTestCase(unittest.TestCase):
 
     target_method = None
 
     def setUp(self):
         self.__seed = testing.generate_seed()
-        self.rs = generator.RandomState(seed=self.__seed)
+        self.rs = _generator.RandomState(seed=self.__seed)
 
     def _get_generator_func(self, *args, **kwargs):
         assert isinstance(self.target_method, str), (
@@ -184,7 +191,7 @@ def _xp_random(xp, method_name):
 class TestRandomState(unittest.TestCase):
 
     def setUp(self):
-        self.rs = generator.RandomState(seed=testing.generate_seed())
+        self.rs = _generator.RandomState(seed=testing.generate_seed())
 
     def check_seed(self, seed):
         rs = self.rs
@@ -221,6 +228,20 @@ class TestRandomState(unittest.TestCase):
     def test_array_seed(self):
         self.check_seed(numpy.random.randint(0, 2**31, size=40))
 
+    def test_methods(self):
+        methods = [
+            cuda.curand.CURAND_RNG_PSEUDO_DEFAULT,
+            cuda.curand.CURAND_RNG_PSEUDO_MRG32K3A,
+            cupy.cuda.curand.CURAND_RNG_PSEUDO_MT19937,
+            cupy.cuda.curand.CURAND_RNG_PSEUDO_PHILOX4_32_10,
+            cupy.cuda.curand.CURAND_RNG_PSEUDO_MTGP32,
+            cupy.cuda.curand.CURAND_RNG_PSEUDO_XORWOW
+        ]
+
+        for method in methods:
+            rs = cupy.random.RandomState(method=method)
+            rs.normal()
+
 
 @testing.parameterize(
     {'a': 1.0, 'b': 3.0},
@@ -449,7 +470,7 @@ class TestLogistic(RandomGeneratorTestCase):
     @condition.repeat(10)
     def test_standard_logistic_isfinite(self):
         x = self.generate(size=10**7)
-        self.assertTrue(cupy.isfinite(x).all())
+        assert cupy.isfinite(x).all()
 
     @testing.for_dtypes('fd')
     @condition.repeat_with_success_at_least(10, 3)
@@ -483,7 +504,7 @@ class TestLogNormal(RandomGeneratorTestCase):
         vals = self.generate_many(
             self.args[0], self.args[1], self.size, dtype, _count=10)
 
-        shape = core.get_size(self.size)
+        shape = _get_size(self.size)
         for val in vals:
             assert isinstance(val, cupy.ndarray)
             assert val.dtype == dtype
@@ -544,7 +565,7 @@ class TestMultivariateNormal(RandomGeneratorTestCase):
             mean=self.args[0], cov=self.args[1], size=self.size, tol=self.tol,
             dtype=dtype, _count=10)
 
-        shape = core.get_size(self.size)
+        shape = _get_size(self.size)
         for val in vals:
             assert isinstance(val, cupy.ndarray)
             assert val.dtype == dtype
@@ -578,7 +599,6 @@ class TestNegativeBinomial(RandomGeneratorTestCase):
     {'df': 2.0, 'nonc': 0.0},
 )
 @testing.gpu
-@testing.with_requires('numpy>=1.11')
 @testing.fix_random()
 class TestNoncentralChisquare(RandomGeneratorTestCase):
 
@@ -634,7 +654,7 @@ class TestNormal(RandomGeneratorTestCase):
         vals = self.generate_many(
             self.args[0], self.args[1], self.size, dtype, _count=10)
 
-        shape = core.get_size(self.size)
+        shape = _get_size(self.size)
         for val in vals:
             assert isinstance(val, cupy.ndarray)
             assert val.dtype == dtype
@@ -724,12 +744,12 @@ class TestStandardT(RandomGeneratorTestCase):
 class TestRandomSample(unittest.TestCase):
 
     def setUp(self):
-        self.rs = generator.RandomState(seed=testing.generate_seed())
+        self.rs = _generator.RandomState(seed=testing.generate_seed())
 
     def check_random_sample(self, dtype):
         vals = [self.rs.random_sample(self.size, dtype) for _ in range(10)]
 
-        shape = core.get_size(self.size)
+        shape = _get_size(self.size)
         for val in vals:
             assert isinstance(val, cupy.ndarray)
             assert val.dtype == dtype
@@ -759,7 +779,7 @@ class TestRandomSampleDistrib(unittest.TestCase):
 class TestRandAndRandN(unittest.TestCase):
 
     def setUp(self):
-        self.rs = generator.RandomState(seed=testing.generate_seed())
+        self.rs = _generator.RandomState(seed=testing.generate_seed())
 
     def test_rand_invalid_argument(self):
         with self.assertRaises(TypeError):
@@ -822,7 +842,7 @@ class TestStandardCauchy(RandomGeneratorTestCase):
     @condition.repeat(10)
     def test_standard_cauchy_isfinite(self):
         x = self.generate(size=10**7)
-        self.assertTrue(cupy.isfinite(x).all())
+        assert cupy.isfinite(x).all()
 
     @testing.for_dtypes('fd')
     @condition.repeat_with_success_at_least(10, 3)
@@ -928,7 +948,7 @@ class TestInterval(RandomGeneratorTestCase):
         vals = [val.get() for val in vals]
         counts = numpy.histogram(vals, bins=numpy.arange(mx + 2))[0]
         expected = numpy.array([float(trial) / (mx + 1)] * (mx + 1))
-        self.assertTrue(hypothesis.chi_square_test(counts, expected))
+        assert hypothesis.chi_square_test(counts, expected)
 
     @condition.repeat(3)
     def test_goodness_of_fit_2(self):
@@ -936,7 +956,7 @@ class TestInterval(RandomGeneratorTestCase):
         vals = self.generate(mx, (5, 5)).get()
         counts = numpy.histogram(vals, bins=numpy.arange(mx + 2))[0]
         expected = numpy.array([float(vals.size) / (mx + 1)] * (mx + 1))
-        self.assertTrue(hypothesis.chi_square_test(counts, expected))
+        assert hypothesis.chi_square_test(counts, expected)
 
 
 @testing.fix_random()
@@ -947,21 +967,21 @@ class TestTomaxint(RandomGeneratorTestCase):
 
     def test_tomaxint_none(self):
         x = self.generate()
-        self.assertEqual(x.shape, ())
-        self.assertTrue((0 <= x).all())
-        self.assertTrue((x <= cupy.iinfo(cupy.int_).max).all())
+        assert x.shape == ()
+        assert (0 <= x).all()
+        assert (x <= cupy.iinfo(cupy.int_).max).all()
 
     def test_tomaxint_int(self):
         x = self.generate(3)
-        self.assertEqual(x.shape, (3,))
-        self.assertTrue((0 <= x).all())
-        self.assertTrue((x <= cupy.iinfo(cupy.int_).max).all())
+        assert x.shape == (3,)
+        assert (0 <= x).all()
+        assert (x <= cupy.iinfo(cupy.int_).max).all()
 
     def test_tomaxint_tuple(self):
         x = self.generate((2, 3))
-        self.assertEqual(x.shape, (2, 3))
-        self.assertTrue((0 <= x).all())
-        self.assertTrue((x <= cupy.iinfo(cupy.int_).max).all())
+        assert x.shape == (2, 3)
+        assert (0 <= x).all()
+        assert (x <= cupy.iinfo(cupy.int_).max).all()
 
 
 @testing.parameterize(
@@ -980,7 +1000,7 @@ class TestChoice1(RandomGeneratorTestCase):
 
     def test_dtype_shape(self):
         v = self.generate(a=self.a, size=self.size, p=self.p)
-        if isinstance(self.size, six.integer_types):
+        if isinstance(self.size, int):
             expected_shape = (self.size,)
         else:
             expected_shape = self.size
@@ -988,8 +1008,8 @@ class TestChoice1(RandomGeneratorTestCase):
             expected_dtype = 'float'
         else:
             expected_dtype = 'int64'
-        self.assertEqual(v.dtype, expected_dtype)
-        self.assertEqual(v.shape, expected_shape)
+        assert v.dtype == expected_dtype
+        assert v.shape == expected_shape
 
     @condition.repeat(3, 10)
     def test_bound(self):
@@ -998,9 +1018,9 @@ class TestChoice1(RandomGeneratorTestCase):
         vals = [val.get() for val in vals]
         size_ = self.size if isinstance(self.size, tuple) else (self.size,)
         for val in vals:
-            self.assertEqual(val.shape, size_)
-        self.assertEqual(min(val.min() for val in vals), 0)
-        self.assertEqual(max(val.max() for val in vals), 2)
+            assert val.shape == size_
+        assert min(val.min() for val in vals) == 0
+        assert max(val.max() for val in vals) == 2
 
 
 @testing.parameterize(
@@ -1014,7 +1034,7 @@ class TestChoice2(RandomGeneratorTestCase):
 
     def test_dtype_shape(self):
         v = self.generate(a=self.a, size=self.size, p=self.p)
-        if isinstance(self.size, six.integer_types):
+        if isinstance(self.size, int):
             expected_shape = (self.size,)
         else:
             expected_shape = self.size
@@ -1022,8 +1042,8 @@ class TestChoice2(RandomGeneratorTestCase):
             expected_dtype = 'float'
         else:
             expected_dtype = 'int'
-        self.assertEqual(v.dtype, expected_dtype)
-        self.assertEqual(v.shape, expected_shape)
+        assert v.dtype == expected_dtype
+        assert v.shape == expected_shape
 
     @condition.repeat(3, 10)
     def test_bound(self):
@@ -1032,9 +1052,9 @@ class TestChoice2(RandomGeneratorTestCase):
         vals = [val.get() for val in vals]
         size_ = self.size if isinstance(self.size, tuple) else (self.size,)
         for val in vals:
-            self.assertEqual(val.shape, size_)
-        self.assertEqual(min(val.min() for val in vals), 0)
-        self.assertEqual(max(val.max() for val in vals), 2)
+            assert val.shape == size_
+        assert min(val.min() for val in vals) == 0
+        assert max(val.max() for val in vals) == 2
 
 
 @testing.fix_random()
@@ -1050,14 +1070,14 @@ class TestChoiceChi(RandomGeneratorTestCase):
         vals = [val.get() for val in vals]
         counts = numpy.histogram(vals, bins=numpy.arange(4))[0]
         expected = numpy.array([30, 30, 40])
-        self.assertTrue(hypothesis.chi_square_test(counts, expected))
+        assert hypothesis.chi_square_test(counts, expected)
 
     @condition.repeat(3, 10)
     def test_goodness_of_fit_2(self):
         vals = self.generate(3, (5, 20), True, [0.3, 0.3, 0.4]).get()
         counts = numpy.histogram(vals, bins=numpy.arange(4))[0]
         expected = numpy.array([30, 30, 40])
-        self.assertTrue(hypothesis.chi_square_test(counts, expected))
+        assert hypothesis.chi_square_test(counts, expected)
 
 
 @testing.fix_random()
@@ -1079,9 +1099,10 @@ class TestChoiceMultinomial(unittest.TestCase):
     {'a': 3.1, 'size': 1, 'p': [0.1, 0.1, 0.8]},
     {'a': None, 'size': 1, 'p': [0.1, 0.1, 0.8]},
     {'a': -3, 'size': 1, 'p': [0.1, 0.1, 0.8]},
-    {'a': [[0, 1], [2]], 'size': 1, 'p': [0.1, 0.1, 0.8]},
+    {'a': [[0, 1], [2, 3]], 'size': 1, 'p': [[0.1, 0.2], [0.3, 0.4]]},
+    {'a': [[0, 1], [2, 3]], 'size': 1, 'p': [0.3, 0.7]},
     {'a': [], 'size': 1, 'p': [0.1, 0.1, 0.8]},
-    {'a': 3, 'size': 1, 'p': [[0.1, 0.1], [0.8]]},
+    {'a': 4, 'size': 1, 'p': [[0.1, 0.2], [0.3, 0.4]]},
     {'a': 2, 'size': 1, 'p': [0.1, 0.1, 0.8]},
     {'a': 3, 'size': 1, 'p': [-0.1, 0.3, 0.8]},
     {'a': 3, 'size': 1, 'p': [0.1, 0.1, 0.7]},
@@ -1091,7 +1112,7 @@ class TestChoiceMultinomial(unittest.TestCase):
 class TestChoiceFailure(unittest.TestCase):
 
     def setUp(self):
-        self.rs = generator.RandomState(seed=testing.generate_seed())
+        self.rs = _generator.RandomState(seed=testing.generate_seed())
 
     def test_choice_invalid_value(self):
         with self.assertRaises(ValueError):
@@ -1112,7 +1133,7 @@ class TestChoiceReplaceFalse(RandomGeneratorTestCase):
 
     def test_dtype_shape(self):
         v = self.generate(a=self.a, size=self.size, replace=False)
-        if isinstance(self.size, six.integer_types):
+        if isinstance(self.size, int):
             expected_shape = (self.size,)
         else:
             expected_shape = self.size
@@ -1120,18 +1141,18 @@ class TestChoiceReplaceFalse(RandomGeneratorTestCase):
             expected_dtype = 'float'
         else:
             expected_dtype = 'int'
-        self.assertEqual(v.dtype, expected_dtype)
-        self.assertEqual(v.shape, expected_shape)
+        assert v.dtype == expected_dtype
+        assert v.shape == expected_shape
 
     @condition.repeat(3, 10)
     def test_bound(self):
         val = self.generate(a=self.a, size=self.size, replace=False).get()
         size = self.size if isinstance(self.size, tuple) else (self.size,)
-        self.assertEqual(val.shape, size)
-        self.assertTrue((0 <= val).all())
-        self.assertTrue((val < 5).all())
+        assert val.shape == size
+        assert (0 <= val).all()
+        assert (val < 5).all()
         val = numpy.asarray(val)
-        self.assertEqual(numpy.unique(val).size, val.size)
+        assert numpy.unique(val).size == val.size
 
 
 @testing.gpu
@@ -1305,18 +1326,19 @@ class TestZipf(RandomGeneratorTestCase):
 @testing.gpu
 class TestChoiceReplaceFalseFailure(unittest.TestCase):
 
-    @testing.numpy_cupy_raises(accept_error=ValueError)
-    def test_choice_invalid_value(self, xp):
-        rs = xp.random.RandomState(seed=testing.generate_seed())
-        rs.choice(a=self.a, size=self.size, replace=False)
+    def test_choice_invalid_value(self):
+        for xp in (numpy, cupy):
+            rs = xp.random.RandomState(seed=testing.generate_seed())
+            with pytest.raises(ValueError):
+                rs.choice(a=self.a, size=self.size, replace=False)
 
 
 class TestResetStates(unittest.TestCase):
 
     def test_reset_states(self):
-        generator._random_states = 'dummy'
-        generator.reset_states()
-        self.assertEqual({}, generator._random_states)
+        _generator._random_states = 'dummy'
+        _generator.reset_states()
+        assert {} == _generator._random_states
 
 
 @testing.gpu
@@ -1324,44 +1346,44 @@ class TestGetRandomState(unittest.TestCase):
 
     def setUp(self):
         self.device_id = cuda.Device().id
-        self.rs_tmp = generator._random_states
+        self.rs_tmp = _generator._random_states
 
     def tearDown(self, *args):
-        generator._random_states = self.rs_tmp
+        _generator._random_states = self.rs_tmp
 
     def test_get_random_state_initialize(self):
-        generator._random_states = {}
-        rs = generator.get_random_state()
-        self.assertEqual(generator._random_states[self.device_id], rs)
+        _generator._random_states = {}
+        rs = _generator.get_random_state()
+        assert _generator._random_states[self.device_id] == rs
 
     def test_get_random_state_memoized(self):
-        generator._random_states = {self.device_id: 'expected',
-                                    self.device_id + 1: 'dummy'}
-        rs = generator.get_random_state()
-        self.assertEqual('expected', generator._random_states[self.device_id])
-        self.assertEqual('dummy', generator._random_states[self.device_id + 1])
-        self.assertEqual('expected', rs)
+        _generator._random_states = {self.device_id: 'expected',
+                                     self.device_id + 1: 'dummy'}
+        rs = _generator.get_random_state()
+        assert 'expected' == _generator._random_states[self.device_id]
+        assert 'dummy' == _generator._random_states[self.device_id + 1]
+        assert 'expected' == rs
 
 
 @testing.gpu
 class TestSetRandomState(unittest.TestCase):
 
     def setUp(self):
-        self.rs_tmp = generator._random_states
+        self.rs_tmp = _generator._random_states
 
     def tearDown(self, *args):
-        generator._random_states = self.rs_tmp
+        _generator._random_states = self.rs_tmp
 
     def test_set_random_state(self):
-        rs = generator.RandomState()
-        generator.set_random_state(rs)
-        assert generator.get_random_state() is rs
+        rs = _generator.RandomState()
+        _generator.set_random_state(rs)
+        assert _generator.get_random_state() is rs
 
     def test_set_random_state_call_multiple_times(self):
-        generator.set_random_state(generator.RandomState())
-        rs = generator.RandomState()
-        generator.set_random_state(rs)
-        assert generator.get_random_state() is rs
+        _generator.set_random_state(_generator.RandomState())
+        rs = _generator.RandomState()
+        _generator.set_random_state(rs)
+        assert _generator.get_random_state() is rs
 
 
 @testing.gpu
@@ -1377,7 +1399,7 @@ class TestStandardExponential(RandomGeneratorTestCase):
     @condition.repeat(10)
     def test_standard_exponential_isfinite(self):
         x = self.generate(size=10**7)
-        self.assertTrue(cupy.isfinite(x).all())
+        assert cupy.isfinite(x).all()
 
     @testing.for_dtypes('fd')
     @condition.repeat_with_success_at_least(10, 3)
@@ -1426,7 +1448,7 @@ class TestRandomStateThreadSafe(unittest.TestCase):
         actual = cupy.random.uniform()
         cupy.random.seed(seed)
         expected = cupy.random.uniform()
-        self.assertEqual(actual, expected)
+        assert actual == expected
 
     def test_set_random_state_thread_safe(self):
         rs = cupy.random.RandomState()
@@ -1447,53 +1469,28 @@ class TestRandomStateThreadSafe(unittest.TestCase):
 class TestGetRandomState2(unittest.TestCase):
 
     def setUp(self):
-        self.rs_dict = generator._random_states
-        generator._random_states = {}
+        self.rs_dict = _generator._random_states
+        _generator._random_states = {}
         self.cupy_seed = os.getenv('CUPY_SEED')
-        self.chainer_seed = os.getenv('CHAINER_SEED')
 
     def tearDown(self, *args):
-        generator._random_states = self.rs_dict
+        _generator._random_states = self.rs_dict
         if self.cupy_seed is None:
             os.environ.pop('CUPY_SEED', None)
         else:
             os.environ['CUPY_SEED'] = self.cupy_seed
-        if self.chainer_seed is None:
-            os.environ.pop('CHAINER_SEED', None)
-        else:
-            os.environ['CHAINER_SEED'] = self.chainer_seed
 
-    def test_get_random_state_no_cupy_no_chainer_seed(self):
+    def test_get_random_state_no_cupy(self):
         os.environ.pop('CUPY_SEED', None)
-        os.environ.pop('CHAINER_SEED', None)
         rvs0 = self._get_rvs_reset()
         rvs1 = self._get_rvs_reset()
 
         self._check_different(rvs0, rvs1)
 
-    def test_get_random_state_no_cupy_with_chainer_seed(self):
-        rvs0 = self._get_rvs(generator.RandomState(5))
-
-        os.environ.pop('CUPY_SEED', None)
-        os.environ['CHAINER_SEED'] = '5'
-        rvs1 = self._get_rvs_reset()
-
-        self._check_same(rvs0, rvs1)
-
-    def test_get_random_state_with_cupy_no_chainer_seed(self):
-        rvs0 = self._get_rvs(generator.RandomState(6))
+    def test_get_random_state_with_cupy(self):
+        rvs0 = self._get_rvs(_generator.RandomState(6))
 
         os.environ['CUPY_SEED'] = '6'
-        os.environ.pop('CHAINER_SEED', None)
-        rvs1 = self._get_rvs_reset()
-
-        self._check_same(rvs0, rvs1)
-
-    def test_get_random_state_with_cupy_with_chainer_seed(self):
-        rvs0 = self._get_rvs(generator.RandomState(7))
-
-        os.environ['CUPY_SEED'] = '7'
-        os.environ['CHAINER_SEED'] = '8'
         rvs1 = self._get_rvs_reset()
 
         self._check_same(rvs0, rvs1)
@@ -1504,8 +1501,8 @@ class TestGetRandomState2(unittest.TestCase):
         return rvu, rvn
 
     def _get_rvs_reset(self):
-        generator.reset_states()
-        return self._get_rvs(generator.get_random_state())
+        _generator.reset_states()
+        return self._get_rvs(_generator.get_random_state())
 
     def _check_same(self, rvs0, rvs1):
         for rv0, rv1 in zip(rvs0, rvs1):
@@ -1514,21 +1511,20 @@ class TestGetRandomState2(unittest.TestCase):
     def _check_different(self, rvs0, rvs1):
         for rv0, rv1 in zip(rvs0, rvs1):
             for r0, r1 in zip(rv0, rv1):
-                self.assertNotEqual(r0, r1)
+                assert r0 != r1
 
 
 class TestCheckAndGetDtype(unittest.TestCase):
 
     @testing.for_float_dtypes(no_float16=True)
     def test_float32_64_type(self, dtype):
-        self.assertEqual(generator._check_and_get_dtype(dtype),
-                         numpy.dtype(dtype))
+        assert _generator._check_and_get_dtype(dtype) == numpy.dtype(dtype)
 
     def test_float16(self):
         with self.assertRaises(TypeError):
-            generator._check_and_get_dtype(numpy.float16)
+            _generator._check_and_get_dtype(numpy.float16)
 
     @testing.for_int_dtypes()
     def test_int_type(self, dtype):
         with self.assertRaises(TypeError):
-            generator._check_and_get_dtype(dtype)
+            _generator._check_and_get_dtype(dtype)
